# -*- coding: utf-8 -*-
"""
Page document views
"""
import json, os, StringIO

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.views import generic
from django.views.generic.detail import SingleObjectMixin
from django.forms.models import modelformset_factory

from braces.views import LoginRequiredMixin, PermissionRequiredMixin

from babel.messages.pofile import read_po, write_po
from babel.messages.catalog import Catalog as BabelCatalog

from po_projects.models import Project, TemplateMsg, Catalog, TranslationMsg
from po_projects.forms import ProjectForm, CatalogForm, TranslationMsgForm
from po_projects.utils import DownloadMixin


class CatalogDetails(LoginRequiredMixin, generic.UpdateView):
    """
    Form view to display Catalog details and edit its infos
    """
    model = Catalog
    template_name = "po_projects/catalog_details.html"
    form_class = CatalogForm

    def get(self, request, *args, **kwargs):
        self.project = self.get_project()
        return super(CatalogDetails, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.project = self.get_project()
        return super(CatalogDetails, self).post(request, *args, **kwargs)

    def get_project(self):
        return get_object_or_404(Project, slug=self.kwargs['slug'])

    def get_object(self):
        return get_object_or_404(Catalog, project=self.project, locale=self.kwargs['locale'])
        
    def get_context_data(self, **kwargs):
        context = super(CatalogDetails, self).get_context_data(**kwargs)
        context.update({
            'project': self.project,
            'catalog': self.object,
        })
        return context

    def get_success_url(self):
        return reverse('po_projects:catalog-details', args=[self.project.slug, self.object.locale])

    def get_form_kwargs(self):
        kwargs = super(CatalogDetails, self).get_form_kwargs()
        kwargs.update({
            'project': self.project,
        })
        return kwargs


def CatalogMessagesFormView(request, slug=None, locale=None):
    """
    Implemented without CBV until i find HOW to do it
    """
    template_name = "po_projects/catalog_messages_form.html"
    
    project = get_object_or_404(Project, slug=slug)
    catalog = get_object_or_404(Catalog, project=project, locale=locale)
    
    formset_queryset = TranslationMsg.objects.select_related('template').filter(catalog=catalog)
    
    TranslationMsgFormSet = modelformset_factory(TranslationMsg, form=TranslationMsgForm, fields=('template','message',), extra=0)
    
    formset = TranslationMsgFormSet(request.POST or None, queryset=formset_queryset)
    
    if formset.is_valid():
        formset.save()
    
    extra_context = {
        "project": project,
        "catalog": catalog,
        "formset": formset
    }
    return render_to_response(template_name, extra_context, context_instance=RequestContext(request))


class CatalogMessagesExportView(LoginRequiredMixin, DownloadMixin, generic.View):
    """
    View to export PO file from a catalog
    """
    content_type = 'text/x-gettext-translation'
    filename_format = "messages_{locale_name}_{timestamp}.po"

    def get(self, request, *args, **kwargs):
        self.project = self.get_project()
        self.object = self.get_object()
        return super(CatalogMessagesExportView, self).get(request, *args, **kwargs)
    
    def get_project(self):
        return get_object_or_404(Project, slug=self.kwargs['slug'])

    def get_object(self):
        return get_object_or_404(Catalog, project=self.project, locale=self.kwargs['locale'])
        
    def get_context_data(self, **kwargs):
        context = super(CatalogMessagesExportView, self).get_context_data(**kwargs)
        context.update({
            'project': self.project,
            'catalog': self.object,
            'locale_name': self.object.locale,
            'timestamp': self.get_filename_timestamp(),
        })
        return context
    
    def get_filename(self, context):
        return self.filename_format.format(**context)
    
    def get_content(self, context):
        forged_catalog = BabelCatalog(
            locale=self.object.locale, 
            header_comment=self.object.header_comment,
            project=self.project.name,
            version=self.project.version
        )
        
        for entry in self.object.translationmsg_set.all().order_by('id'):
            locations = [tuple(item) for item in json.loads(entry.template.locations)]
            forged_catalog.add(entry.template.message, string=entry.message, locations=locations, flags=entry.template.flags)
            
        fpw = StringIO.StringIO()
        write_po(fpw, forged_catalog, sort_by_file=False, ignore_obsolete=True, include_previous=False)
        return fpw.getvalue()



# Import from django
from django.contrib import admin

# Import from here
from .models import Project, TemplateMsg, Catalog, TranslationMsg

admin.site.register(Project)
admin.site.register(TemplateMsg)
admin.site.register(Catalog)
admin.site.register(TranslationMsg)



import datetime

from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect, Http404

class DownloadMixin(object):
    """
    Simple Mixin to send a downloadable content
    
    Inherits must have :
    
    * Filled the ``self.content_type`` attribute with the content content_type to send;
    * Implementation of ``get_filename()`` that return the filename to use in response 
      headers;
    * Implementation of ``get_content()`` that return the content to send as downloadable.
    
    If the content is a not a string, it is assumed to be a fileobject to send as 
    the content with its ``read()`` method.
    
    Optionnaly implement a ``close_content()`` to close specifics objects linked to 
    content fileobject, if it does not exists a try will be made on a close() method 
    on the content fileobject;
    
    A "get_filename_timestamp" method is implemented to return a timestamp to use in your 
    filename if needed, his date format is defined in "timestamp_format" attribute (in a 
    suitable way to use with strftime on a datetime object).
    """
    content_type = None
    timestamp_format = "%Y-%m-%d"
    
    def get_filename_timestamp(self):
        return datetime.datetime.now().strftime(self.timestamp_format)
    
    def get_filename(self, context):
        raise ImproperlyConfigured("DownloadMixin requires an implementation of 'get_filename()' to return the filename to use in headers")
    
    def get_content(self, context):
        raise ImproperlyConfigured("DownloadMixin requires an implementation of 'get_content()' to return the downloadable content")
    
    def render_to_response(self, context, **response_kwargs):
        if getattr(self, 'content_type', None) is None:
            raise ImproperlyConfigured("DownloadMixin requires a definition of 'content_type' attribute")
        # Needed headers
        response = HttpResponse(content_type=self.content_type, **response_kwargs)
        response['Content-Disposition'] = 'attachment; filename={0}'.format(self.get_filename(context))
        # Read the content file object or string, append it to response and close it
        content = self.get_content(context)
        if isinstance(content, basestring):
            response.write(content)
        else:
            response.write(content.read())
        # Conditionnal closing content object
        if hasattr(self, 'close_content'):
            self.close_content(context, content)
        elif hasattr(content, 'close'):
            content.close()
            
        return response

    def get_context_data(self, **kwargs):
        return {
            'params': kwargs
        }

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)



# -*- coding: utf-8 -*-
"""
Page document views
"""
import json, os
from cStringIO import StringIO

from django.conf import settings
from django.core.urlresolvers import reverse
from django.http import HttpResponse, Http404
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.views import generic
from django.views.generic.detail import SingleObjectMixin
from django.forms.models import modelformset_factory

from braces.views import LoginRequiredMixin, PermissionRequiredMixin

from babel.messages.pofile import read_po, write_po
from babel.messages.catalog import Catalog as BabelCatalog

from po_projects.models import Project, TemplateMsg, Catalog, TranslationMsg
from po_projects.forms import ProjectForm, CatalogForm, TranslationMsgForm
from po_projects.utils import DownloadMixin
from po_projects.dump import po_project_export

class ProjectIndex(generic.TemplateView):
    """
    Project index
    """
    template_name = "po_projects/project_index.html"
    
    def get(self, request, *args, **kwargs):
        context = {'project_list' : Project.objects.all().order_by('name')}
        return self.render_to_response(context)


#class ProjectCreateView(LoginRequiredMixin, PermissionRequiredMixin, generic.CreateView):
class ProjectCreateView(LoginRequiredMixin, generic.CreateView):
    """
    Form view to create a Project
    """
    model = Project
    template_name = "po_projects/project_form.html"
    form_class = ProjectForm

    def get_success_url(self):
        return reverse('po_projects:project-details', args=[self.object.slug])

    def get_form_kwargs(self):
        kwargs = super(ProjectCreateView, self).get_form_kwargs()
        kwargs.update({
            'author': self.request.user,
        })
        return kwargs


class ProjectDetails(LoginRequiredMixin, generic.CreateView):
    """
    Form view to display Project details and append a new Catalog
    """
    model = Catalog
    template_name = "po_projects/project_details.html"
    form_class = CatalogForm

    def get(self, request, *args, **kwargs):
        self.project = self.get_project(**kwargs)
        return super(ProjectDetails, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.project = self.get_project(**kwargs)
        return super(ProjectDetails, self).post(request, *args, **kwargs)

    def get_project(self, **kwargs):
        return get_object_or_404(Project, slug=kwargs['slug'])
        
    def get_context_data(self, **kwargs):
        context = super(ProjectDetails, self).get_context_data(**kwargs)
        context.update({
            'project': self.project,
        })
        return context

    def get_success_url(self):
        return reverse('po_projects:project-details', args=[self.project.slug])

    def get_form_kwargs(self):
        kwargs = super(ProjectDetails, self).get_form_kwargs()
        kwargs.update({
            'project': self.project,
        })
        return kwargs


class ProjectExportView(LoginRequiredMixin, DownloadMixin, generic.View):
    """
    View to export PO catalog files from a project into a gzip tarball
    """
    content_type = 'application/x-gzip'
    filename_format = "{project_slug}_{timestamp}.tar.gz"

    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(ProjectExportView, self).get(request, *args, **kwargs)

    def get_object(self):
        return get_object_or_404(Project, slug=self.kwargs['slug'])

    def get_context_data(self, **kwargs):
        context = super(ProjectExportView, self).get_context_data(**kwargs)
        context.update({
            'project': self.object,
            'project_slug': self.object.slug,
            'timestamp': self.get_filename_timestamp(),
        })
        return context
    
    def get_filename(self, context):
        return self.filename_format.format(**context)
    
    def get_content(self, context):
        archive_file = StringIO()
        
        # TODO: give directly the project instance not its slug
        po_project_export(self.object.slug, archive_file)
        
        #archive_file.close()
        return archive_file.getvalue()



# -*- coding: utf-8 -*-
"""
Forms for po_projects
"""
import json

from django.conf import settings
from django import forms
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _

from crispy_forms.helper import FormHelper
from crispy_forms.bootstrap import UneditableField
from crispy_forms_foundation.layout import Layout, Fieldset, Row, Column, ButtonHolder, Submit

from babel import Locale
from babel.core import UnknownLocaleError, get_locale_identifier
from babel.messages.pofile import read_po

from po_projects.models import Project, TemplateMsg, Catalog, TranslationMsg

class ProjectForm(forms.ModelForm):
    """Project Form"""
    po_file = forms.FileField(label=_('PO File'), required=True, help_text='Upload a valid PO file to initialize project strings to translate')
    
    def __init__(self, author=None, *args, **kwargs):
        self.catalog = None
        self.catalog_entries = []
        
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.add_input(Submit('submit', 'Submit'))

        super(ProjectForm, self).__init__(*args, **kwargs)

    def clean_po_file(self):
        data = self.cleaned_data['po_file']
        if data:
            try:
                self.catalog = read_po(data, ignore_obsolete=True)
            except:
                raise forms.ValidationError("Your file does not seem to be a valid PO file")

        return data

    def save(self, commit=True):
        project = super(ProjectForm, self).save(commit=False)
        project.version = "0.1.0"
        
        if commit:
            project.header_comment = self.catalog.header_comment
            project.mime_headers = json.dumps(dict(self.catalog.mime_headers))
            project.save()
            
            entries = []
            for message in self.catalog:
                if message.id:
                    flags = message.flags
                    if flags == set([]):
                        flags = ''
                    entries.append(TemplateMsg(project=project, message=message.id, locations=json.dumps(message.locations), flags=flags))
                
            TemplateMsg.objects.bulk_create(entries)
            
        return project

    class Meta:
        model = Project
        exclude = ('version', 'header_comment', 'mime_headers')

class CatalogForm(forms.ModelForm):
    """Catalog Form"""
    def __init__(self, project=None, *args, **kwargs):
        self.project = project
        
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.add_input(Submit('submit', 'Submit'))

        super(CatalogForm, self).__init__(*args, **kwargs)

    def clean_locale(self):
        data = self.cleaned_data['locale']
        if data:
            # only accept "_" as separator, all "-" are replaced to "_"
            data = "_".join(data.split("-"))
            try:
                self.locale_trans = Locale.parse(data)
            except UnknownLocaleError:
                raise forms.ValidationError("Invalid locale")
            except ValueError:
                raise forms.ValidationError("Invalid locale")
            else:
                data = get_locale_identifier((self.locale_trans.language, self.locale_trans.territory, self.locale_trans.script, self.locale_trans.variant), sep='_')
        return data

    def save(self, commit=True):
        catalog = super(CatalogForm, self).save(commit=False)
        catalog.project = self.project
        
        if commit:
            catalog.header_comment = self.project.header_comment
            catalog.mime_headers = self.project.mime_headers
            catalog.save()
            
            # Fill catalog with template messages
            entries = []
            for row in self.project.templatemsg_set.all():
                entries.append(TranslationMsg(template=row, catalog=catalog, message=''))
            
            TranslationMsg.objects.bulk_create(entries)
            
        return catalog

    class Meta:
        model = Catalog
        exclude = ('project', 'header_comment', 'mime_headers')

class SourceTextField(UneditableField):
    """
    Layout object for rendering template field as simple html text
    """
    template = "po_projects/input_as_text.html"

class TranslationMsgForm(forms.ModelForm):
    """Translation Form"""
    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_action = '.'
        self.helper.layout = Layout(
            Row(
                Column(
                    SourceTextField('template'),
                    css_class='small-6'
                ),
                Column(
                    'message',
                    css_class='small-6'
                ),
            ),
        )
        self.helper.add_input(Submit('submit', 'Submit'))

        super(TranslationMsgForm, self).__init__(*args, **kwargs)

    def save(self, commit=True):
        message = super(TranslationMsgForm, self).save(commit=False)
        
        if commit:
            message.save()
            
        return message

    class Meta:
        model = TranslationMsg
        exclude = ('catalog',)
        widgets = {
            'message': forms.Textarea(attrs={'rows': 3}),
        }




# -*- coding: utf-8 -*-
"""
Just a command line to test some things in development
"""
from cStringIO import StringIO
import json, os, tarfile, time

from optparse import OptionValueError, make_option

from babel.messages.pofile import read_po, write_po
from babel import Locale
from babel.core import UnknownLocaleError, get_locale_identifier
from babel.messages.catalog import Catalog as BabelCatalog

from django.conf import settings
from django.core.cache import cache
from django.core.management.base import CommandError, BaseCommand

from po_projects.models import Project, Catalog, TranslationMsg
from po_projects.dump import po_project_export

PO_ARCHIVE_PATH = "locale/{locale}/LC_MESSAGES/messages.po"

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        make_option("--source", dest="source_filepath", default=None, help="Source to import."),
        make_option("--project", dest="project_slug", default=None, help="Project slug to export"),
    )
    help = "PO Project CLI"

    def handle(self, *args, **options):
        if len(args) != 0:
            raise CommandError("Command doesn't accept any arguments")
        
        self.source_filepath = options.get('source_filepath')
        self.project_slug = options.get('project_slug')
        self.verbosity = int(options.get('verbosity'))
        
        self.just_do_it()

    def just_do_it(self):
        self.do_po_project_export()

    def do_po_project_export(self):
        """
        Export all catalogs from a project into PO files with the good directory 
        structure
        """
        if not self.project_slug:
            raise CommandError("project slug is empty")
        
        # Open and fill tarball archive
        #archive_file = StringIO()
        archive_file = open("{0}.tar.gz".format(self.project_slug), "wb")
        
        po_project_export(self.project_slug, archive_file)
        
        archive_file.close()

    def po_catalog_export(self):
        """
        Export a catalog from a project into a PO file
        """
        toast_path = "/home/django/Emencia/po_headquarter/dummy_po/"
        
        project = Project.objects.get(slug='dummy')
        catalog = Catalog.objects.get(project=project, locale='fr')
        
        #mime_dict = json.loads(catalog.mime_headers)
        
        forged_catalog = BabelCatalog(
            locale=catalog.locale, 
            header_comment=catalog.header_comment,
            project=project.name,
            version="0.2.0"
        )
        
        print "before add:", len(forged_catalog)
        
        for entry in catalog.translationmsg_set.all().order_by('id'):
            locations = [tuple(item) for item in json.loads(entry.template.locations)]
            forged_catalog.add(entry.template.message, string=entry.message, locations=locations, flags=entry.template.flags)
        
        print "after add:", len(forged_catalog)
        print "errors:", [item for item in forged_catalog.check()]
        
        print
        print "---------------- Original"
        fpw = StringIO()
        write_po(fpw, forged_catalog, sort_by_file=False, ignore_obsolete=True, include_previous=False)
        print fpw.getvalue()
        fpw.close()
        
        print
        print "---------------- Updated"
        fp3 = open(os.path.join(toast_path, '0-3-0.pot'), 'r')
        template_catalog_3 = read_po(fp3)
        forged_catalog.update(template_catalog_3)
        
        fpw = StringIO()
        write_po(fpw, forged_catalog, sort_by_file=False, ignore_obsolete=True, include_previous=False)
        print fpw.getvalue()
        fpw.close()

    def po_update(self):
        """
        Testing PO file update with another PO file
        """
        toast_path = "/home/django/Emencia/po_headquarter/dummy_po/"
        
        project = Project.objects.get(slug='dummy')
        catalog = Catalog.objects.get(project=project, locale='fr')
        
        fp1 = open(os.path.join(toast_path, '0-1-0.pot'), 'r')
        fp2 = open(os.path.join(toast_path, '0-2-0.pot'), 'r')
        
        template_catalog_1 = read_po(fp1)
        template_catalog_2 = read_po(fp2)
        
        #print "before update:", len(catalog)
        template_catalog_1.update(template_catalog_2)
        #print "after update:", len(catalog)
        
        print catalog

    def locale_tuple(self):
        """
        Testing locale identifier parsing attributes
        """
        l = Locale.parse('en-AU')
        print "language:", l.language
        print "territory:", l.territory
        print "script:", l.script
        print "variant:", l.variant
        
        print get_locale_identifier((l.language, l.territory, l.script, l.variant), sep='_')

    def check_locale(self):
        """
        Testing locale identifier parsing with error
        """
        print Locale.parse('fr')
        try:
            print Locale.parse('francais')
        except ValueError:
            print 'ValueError!'
        except UnknownLocaleError:
            print 'UnknownLocaleError!'
        print Locale.parse('zh_CN')
        print Locale.parse('en_AU')

    def open_po(self):
        """
        Testing PO file opening with --source CLI argument
        """
        if self.source_filepath:
            fp = open(self.source_filepath, "r")
            try:
                catalog = read_po(fp)
            except:
                raise CommandError("Invalid PO file")
            else:
                print catalog.project
                print json.dumps(catalog.mime_headers, indent=4)
            finally:
                fp.close()
        else:
            raise CommandError("Command action need the --source argument")



# -*- coding: utf-8 -*-
"""
Models for po_projects


> Project
    |
    |____> TemplateMsg
    |____> TemplateMsg
    |____> TemplateMsg
    |
    |____> Catalog (fr)
            |
            |___> TranslationMsg
            |___> TranslationMsg
            |___> TranslationMsg
    |
    |____> Catalog (zh_hk)
            |
            |___> TranslationMsg
            |___> TranslationMsg
            |___> TranslationMsg

"""
from babel.core import Locale
from django.db import models
from django.utils.translation import ugettext_lazy as _

class Project(models.Model):
    """
    Project contains catalog and template catalog
    """
    name = models.CharField(_('name'), max_length=150)
    slug = models.SlugField(_('slug'), unique=True, max_length=75)
    version = models.CharField(_('version'), max_length=15)
    header_comment = models.TextField(_('header comment'))
    mime_headers = models.TextField(_('mime headers'))

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('project')
        verbose_name_plural = _('projects')

class TemplateMsg(models.Model):
    """
    Template catalog item, equivalent to a msg from a POT file
    """
    project = models.ForeignKey(Project, verbose_name=_('project'), blank=False)
    message = models.TextField(_('message'), blank=False)
    locations = models.TextField(_('locations'))
    flags = models.TextField(_('flags'))

    def __unicode__(self):
        return self.message

    class Meta:
        verbose_name = _('template message')
        verbose_name_plural = _('templates messages')

class Catalog(models.Model):
    """
    Language catalog for a project, the PO file equivalent
    """
    project = models.ForeignKey(Project, verbose_name=_('project'), blank=False)
    locale = models.CharField(_('locale'), max_length=50, blank=False)
    header_comment = models.TextField(_('header comment'))
    mime_headers = models.TextField(_('mime headers'))

    def __unicode__(self):
        l = Locale.parse(self.locale)
        return l.english_name

    class Meta:
        verbose_name = _('catalog')
        verbose_name_plural = _('catalogs')

class TranslationMsg(models.Model):
    """
    Translation message from a catalog
    """
    template = models.ForeignKey(TemplateMsg, verbose_name=_('row source'), blank=False)
    catalog = models.ForeignKey(Catalog, verbose_name=_('catalog'), blank=False)
    message = models.TextField(_('message'), blank=True)
    # TODO: add fuzzy boolean field

    def __unicode__(self):
        return self.message

    class Meta:
        verbose_name = _('translation message')
        verbose_name_plural = _('translations messages')



# -*- coding: utf-8 -*-
"""
Root url's map for application
"""
from django.conf.urls.defaults import *

from po_projects.views.projects import ProjectIndex, ProjectCreateView, ProjectDetails, ProjectExportView
from po_projects.views.catalogs import CatalogDetails, CatalogMessagesExportView

urlpatterns = patterns('',
    url(r'^$', ProjectIndex.as_view(), name='project-index'),
    url(r'^create/$', ProjectCreateView.as_view(), name='project-create'),
    url(r'^(?P<slug>[-\w]+)/$', ProjectDetails.as_view(), name='project-details'),
    url(r'^(?P<slug>[-\w]+)/download/$', ProjectExportView.as_view(), name='project-download'),
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/$', CatalogDetails.as_view(), name='catalog-details'),
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/edit/$', "po_projects.views.catalogs.CatalogMessagesFormView", name='catalog-messages-edit'),
    url(r'^(?P<slug>[-\w]+)/(?P<locale>[-\w]+)/download/$', CatalogMessagesExportView.as_view(), name='catalog-messages-download'),
)



# -*- coding: utf-8 -*-
"""
Dump tools
"""
from cStringIO import StringIO
import json, os, tarfile, time

from babel.messages.pofile import write_po
from babel import Locale
from babel.core import UnknownLocaleError, get_locale_identifier
from babel.messages.catalog import Catalog as BabelCatalog

from django.conf import settings

from po_projects.models import Project, Catalog

PO_ARCHIVE_PATH = "locale/{locale}/LC_MESSAGES/messages.po"

def po_project_export(project_slug, archive_fileobj):
    """
    Export all catalogs from a project into PO files with the good directory 
    structure
    
    TODO: accept a project slug OR a project instance (to avoid to get it again from database)
    """
    #print "Project :", project_slug
    project = Project.objects.get(slug=project_slug)
    #print "- name :", project.name
    
    archive_files = []
    
    for catalog in project.catalog_set.all():
        archived_path = PO_ARCHIVE_PATH.format(locale=catalog.locale)
        #print " * Catalog:", catalog.locale
        #print "     - Path :", archived_path
        # Open a new catalog
        babel_catalog = BabelCatalog(
            locale=catalog.locale, 
            header_comment=catalog.header_comment,
            project=project.name,
            version=project.version
        )
        # Add its entries
        for entry in catalog.translationmsg_set.all().order_by('id'):
            locations = [tuple(item) for item in json.loads(entry.template.locations)]
            babel_catalog.add(entry.template.message, string=entry.message, locations=locations, flags=entry.template.flags)
        # Write it to a buffer string
        catalog_file = StringIO()
        write_po(catalog_file, babel_catalog, sort_by_file=False, ignore_obsolete=True, include_previous=False)
        catalog_file.seek(0)
        
        archive_files.append( (archived_path, catalog_file) )
    
    # Open and fill tarball archive
    # TODO: restore fileobj usage
    archive = tarfile.open("{0}.tar.gz".format(project.slug), mode="w:gz", fileobj=archive_fileobj)
    mtime = time.time()
    
    for name,content in archive_files:
        info = tarfile.TarInfo(name)
        info.size=len(content.getvalue())
        info.mtime = mtime
        archive.addfile(tarinfo=info, fileobj=content)
        #
        content.close()
    
    archive.close()
    
    return archive_fileobj

